from easy_spider.core.env import async_env
from easy_spider.core.spider import AsyncSpider
from easy_spider.network.request import Request
from easy_spider.network.response import Response, HTMLResponse, TextResponse

__version__ = "1.0.3"
name = "easy_spider"
